@@ -17,7 +17,6 @@ class Gitlab(object):
         """
         on init we setup the token used for all the api calls and all the urls
         :param host: host of gitlab
-        :param user: user
         :param token: token
         """
         if token != "":
@@ -42,10 +41,12 @@ class Gitlab(object):
         """
         data = {"email": user, "password": password}
         request = requests.post(self.host + "/api/v3/session", data=data, 
-                                    verify=self.verify_ssl)
+                                verify=self.verify_ssl,
+                                headers={"connection": "close"})
         if request.status_code == 201:
-            self.token = json.loads((request.content).decode("utf-8"))['private_token']
-            self.headers = {"PRIVATE-TOKEN": self.token}
+            self.token = json.loads(request.content.decode("utf-8"))['private_token']
+            self.headers = {"PRIVATE-TOKEN": self.token,
+                            "connection": "close"}
             return True
         else:
             return False
@@ -61,7 +62,7 @@ class Gitlab(object):
         request = requests.get(self.users_url, params=data,
                                headers=self.headers, verify=self.verify_ssl)
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             return False
 
@@ -93,9 +94,9 @@ class Gitlab(object):
         if sudo != "":
             data['sudo'] = sudo
         request = requests.post(self.users_url, headers=self.headers, data=data, 
-                                    verify=self.verify_ssl)
+                                verify=self.verify_ssl)
         if request.status_code == 201:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         elif request.status_code == 404:
             return False
 
@@ -123,7 +124,7 @@ class Gitlab(object):
         """
         request = requests.get(self.host + "/api/v3/user",
                                headers=self.headers, verify=self.verify_ssl)
-        return json.loads((request.content).decode("utf-8"))
+        return json.loads(request.content.decode("utf-8"))
 
     def edituser(self, id_,
                  name="",
@@ -197,10 +198,10 @@ class Gitlab(object):
         Gets all the ssh keys for the current user
         :return: a dictionary with the lists
         """
-        request = requests.get(self.keys_url, headers=self.headers, 
-                                verify=self.verify_ssl)
+        request = requests.get(self.keys_url, headers=self.headers,
+                               verify=self.verify_ssl)
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             
             return False
@@ -214,7 +215,7 @@ class Gitlab(object):
         request = requests.get(self.keys_url + "/" + str(id_),
                                headers=self.headers, verify=self.verify_ssl)
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             
             return False
@@ -231,7 +232,7 @@ class Gitlab(object):
         if sudo != "":
             data['sudo'] = sudo
         request = requests.post(self.keys_url, headers=self.headers, data=data, 
-                                    verify=self.verify_ssl)
+                                verify=self.verify_ssl)
         if request.status_code == 201:
             return True
         else:
@@ -286,7 +287,7 @@ class Gitlab(object):
         request = requests.get(self.projects_url, params=data,
                                headers=self.headers, verify=self.verify_ssl)
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             
             return False
@@ -300,7 +301,7 @@ class Gitlab(object):
         request = requests.get(self.projects_url + "/" + str(id_),
                                headers=self.headers, verify=self.verify_ssl)
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             
             return False
@@ -317,9 +318,9 @@ class Gitlab(object):
         data = {'page': page, 'per_page': per_page}
         request = requests.get(self.projects_url + "/" + str(id_) +
                                "/events", params=data, headers=self.headers, 
-                                    verify=self.verify_ssl)
+                               verify=self.verify_ssl)
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             
             return False
@@ -352,7 +353,7 @@ class Gitlab(object):
         request = requests.post(self.projects_url, headers=self.headers,
                                 data=data, verify=self.verify_ssl)
         if request.status_code == 201:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         elif request.status_code == 403:
             if "Your own projects limit is 0" in request.content:
                 print(request.content)
@@ -396,7 +397,7 @@ class Gitlab(object):
         request = requests.get(self.projects_url + "/" + str(id_) + "/members",
                                headers=self.headers, verify=self.verify_ssl)
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             
             return False
@@ -480,7 +481,7 @@ class Gitlab(object):
         request = requests.get(self.projects_url + "/" + str(id_) + "/hooks",
                                headers=self.headers, verify=self.verify_ssl)
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             
             return False
@@ -495,7 +496,7 @@ class Gitlab(object):
         request = requests.get(self.projects_url + "/" + str(id_) + "/hooks/" +
                                str(hook_id), headers=self.headers, verify=self.verify_ssl)
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             
             return False
@@ -560,10 +561,10 @@ class Gitlab(object):
         :return: the branches
         """
         request = requests.get(self.projects_url + "/" + str(id_) +
-                               "/repository/branches", headers=self.headers, 
-                                verify=self.verify_ssl)
+                               "/repository/branches", headers=self.headers,
+                               verify=self.verify_ssl)
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             return False
 
@@ -578,7 +579,7 @@ class Gitlab(object):
                                "/repository/branches/" + str(branch),
                                headers=self.headers, verify=self.verify_ssl)
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             
             return False
@@ -591,9 +592,8 @@ class Gitlab(object):
         :return: True if success
         """
         request = requests.put(self.projects_url + "/" + str(id_) +
-                               "/repository/branches/" + str(
-            branch) + "/protect",
-                               headers=self.headers)
+                               "/repository/branches/" + str(branch) +
+                               "/protect", headers=self.headers)
         if request.status_code == 200:
             return True
         else:
@@ -608,9 +608,8 @@ class Gitlab(object):
         :return: true if success
         """
         request = requests.put(self.projects_url + "/" + str(id_) +
-                               "/repository/branches/" + str(
-            branch) + "/unprotect",
-                               headers=self.headers)
+                               "/repository/branches/" + str(branch) +
+                               "/unprotect", headers=self.headers)
         if request.status_code == 200:
             return True
         else:
@@ -662,7 +661,7 @@ class Gitlab(object):
         request = requests.get(self.host + "/api/v3/issues",
                                params=data, headers=self.headers, verify=self.verify_ssl)
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             
             return False
@@ -678,10 +677,10 @@ class Gitlab(object):
         if sudo != "":
             data['sudo'] = sudo
         request = requests.get(self.projects_url + "/" + str(id_) +
-                               "/issues", params=data, headers=self.headers, 
-                                verify=self.verify_ssl)
+                               "/issues", params=data, headers=self.headers,
+                               verify=self.verify_ssl)
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             
             return False
@@ -697,7 +696,7 @@ class Gitlab(object):
                                "/issues/" + str(issue_id), headers=self.headers, 
                                verify=self.verify_ssl)
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             
             return False
@@ -776,7 +775,7 @@ class Gitlab(object):
         request = requests.get(self.projects_url + "/" + str(id_) +
                                "/milestones", headers=self.headers, verify=self.verify_ssl)
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             
             return False
@@ -792,7 +791,7 @@ class Gitlab(object):
                                + "/milestones/"
                                + str(milestone_id), headers=self.headers, verify=self.verify_ssl)
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             
             return False
@@ -857,7 +856,7 @@ class Gitlab(object):
         request = requests.get(self.projects_url + "/" + str(id_) + "/keys",
                                headers=self.headers, verify=self.verify_ssl)
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             
             return False
@@ -872,7 +871,7 @@ class Gitlab(object):
         request = requests.get(self.projects_url + "/" + str(id_) + "/keys/" +
                                str(key_id), headers=self.headers, verify=self.verify_ssl)
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             
             return False
@@ -963,7 +962,7 @@ class Gitlab(object):
                                params=data, headers=self.headers,
                                verify=self.verify_ssl)
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             
             return False
@@ -996,10 +995,10 @@ class Gitlab(object):
             data['sudo'] = sudo
         url_str = '{0}/{1}/merge_requests'.format(self.projects_url, project_id)
         request = requests.get(url_str, params=data, headers=self.headers,
-                                        verify=self.verify_ssl)
+                               verify=self.verify_ssl)
 
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             
             return False
@@ -1017,7 +1016,7 @@ class Gitlab(object):
         request = requests.get(url_str, headers=self.headers, verify=self.verify_ssl)
 
         if request.status_code == 200:
-            return json.loads((request.content).decode("utf-8"))
+            return json.loads(request.content.decode("utf-8"))
         else:
             
             return False
@@ -1034,14 +1033,14 @@ class Gitlab(object):
         """
         url_str = '{0}/{1}/merge_requests'.format(self.projects_url, project_id)
         data = {'source_branch': sourcebranch,
-                  'target_branch': targetbranch,
-                  'title': title,
-                  'assignee_id': assignee_id}
+                'target_branch': targetbranch,
+                'title': title,
+                'assignee_id': assignee_id}
         if sudo != "":
             data['sudo'] = sudo
 
         request = requests.post(url_str, data=data, headers=self.headers, 
-                                         verify=self.verify_ssl)
+                                verify=self.verify_ssl)
         if request.status_code == 201:
             return True
         else:
@@ -1065,15 +1064,15 @@ class Gitlab(object):
                                                      project_id,
                                                      mergerequest_id)
         data = {'source_branch': sourcebranch,
-                  'target_branch': targetbranch,
-                  'title': title,
-                  'assignee_id': assignee_id,
-                  'closed': closed}
+                'target_branch': targetbranch,
+                'title': title,
+                'assignee_id': assignee_id,
+                'closed': closed}
         if sudo != "":
             data['sudo'] = sudo
 
         request = requests.post(url_str, data=data, headers=self.headers, 
-                                         verify=self.verify_ssl)
+                                verify=self.verify_ssl)
         if request.status_code == 201:
             return True
         else:
@@ -1098,3 +1097,169 @@ class Gitlab(object):
         else:
             
             return False
+
+    def getsnippets(self, project_id):
+        """
+        Get all the snippets of the project identified by project_id
+        @param project_id: project id to get the snippets from
+        @return: list of dictionaries
+        """
+        request = requests.get(self.projects_url + "/" + str(project_id) + "/snippets",
+                               headers=self.headers)
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
+            return False
+
+    def getsnippet(self, project_id, snippet_id):
+        """
+        Get one snippet from a project
+        @param project_id: project id to get the snippet from
+        @param snippet_id: snippet id
+        @return: dictionary
+        """
+        request = requests.get(self.projects_url + "/" + str(project_id) + "/snippets/" + str(snippet_id),
+                               headers=self.headers)
+        if request.status_code == 200:
+            return json.loads(request.content.decode('utf-8'))
+        else:
+            return False
+
+    def createsnippet(self, project_id, title, file_name, code, lifetime=""):
+        """
+        Creates an snippet
+        @param project_id: project id to create the snippet under
+        @param title: title of the snippet
+        @param file_name: filename for the snippet
+        @param code: content of the snippet
+        @param lifetime: expiration date
+        @return: True if correct, false if failed
+        """
+        data = {"id": project_id, "title": title, "file_name": file_name, "code": code}
+        if lifetime != "":
+            data["lifetime"] = lifetime
+        request = requests.post(self.projects_url + "/" + str(project_id) + "/snippets",
+                                data=data, headers=self.headers)
+        if request.status_code == 201:
+            return True
+        else:
+            return False
+
+    def getsnippetcontent(self, project_id, snippet_id):
+        request = requests.get(self.projects_url + "/" + str(project_id) +
+                               "/snippets/" + str(snippet_id) + "/raw",
+                               headers=self.headers)
+        if request.status_code == 200:
+            return request.content
+        else:
+            return False
+
+    def deletesnippet(self, project_id, snippet_id):
+        request = requests.delete(self.projects_url + "/" + str(project_id) +
+                                  "/snippets/" + str(snippet_id), headers=self.headers)
+        if request.status_code == 200:
+            return True
+        else:
+            return False
+
+    def getrepositories(self, project_id):
+        request = requests.get(self.projects_url + "/" + str(project_id) + "/repository/branches",
+                               headers=self.headers)
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
+            return False
+
+    def getrepositorybranch(self, project_id, branch):
+        request = requests.get(self.projects_url + "/" + str(project_id) +
+                               "/repository/branches/" + str(branch),
+                               headers=self.headers)
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        elif request.status_code == 404:
+            if json.loads(request.content.decode("utf-8"))['message'] == "404 Branch does not exist Not Found":
+                # In the future we should raise an exception here
+                return False
+        else:
+            return False
+
+    def protectrepositorybranch(self, project_id, branch_name):
+        request = requests.put(self.projects_url + "/" + str(project_id) +
+                               "/repository/branches/" + str(branch_name) + "/protect",
+                               headers=self.headers)
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
+            return False
+
+    def unprotectrepositorybranch(self, project_id, branch_name):
+        request = requests.put(self.projects_url + "/" + str(project_id) +
+                               "/repository/branches/" + str(branch_name) + "/unprotect",
+                               headers=self.headers)
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
+            return
+
+    def listrepositorytags(self, project_id):
+        request = requests.get(self.projects_url + "/" + str(project_id) +
+                               "/repository/tags", headers=self.headers)
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
+            return False
+
+    def listrepositorycommits(self, project_id):
+        request = requests.get(self.projects_url + "/" + str(project_id) +
+                               "/repository/commits", headers=self.headers)
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
+            return False
+
+    def listrepositorycommit(self, project_id, sha1):
+        request = requests.get(self.projects_url + "/" + str(project_id) +
+                               "/repository/commits/" + str(sha1),
+                               headers=self.headers)
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
+            return False
+
+    def listrepositorycommitdiff(self, project_id, sha1):
+        request = requests.get(self.projects_url + "/" + str(project_id) +
+                               "/repository/commits/" + str(sha1) + "/diff",
+                               headers=self.headers)
+        if request.status_code == 200:
+            # it returns a list of dicts, which is nonsense as we are requesting
+            # just one diff, so we use the [0] to return only the first and only
+            # element
+            return json.loads(request.content.decode("utf-8"))[0]
+        else:
+            return False
+
+    def listrepositorytree(self, project_id, path="", ref_name=""):
+        data = {}
+        if path != "":
+            data['path'] = path
+        if ref_name != "":
+            data['ref_name'] = ref_name
+
+        request = requests.get(self.projects_url + "/" + str(project_id) +
+                               "/repository/tree/", data=data,
+                               headers=self.headers)
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
+            return False
+
+    def getrawblob(self, project_id, sha1, path):
+        data = {"filepath": path}
+
+        request = requests.get(self.projects_url + "/" + str(project_id) +
+                               "/repository/blobs/" + str(sha1),
+                               data=data, headers=self.headers)
+        if request.status_code == 200:
+            return request.content.decode("utf-8")
+        else:
+            return False
